using System;
using System.Collections.Generic;
using System.Drawing;
using System.Text;
using System.Web.UI.Design;
using System.Web.UI.Design.WebControls;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Drawing.Design;
using System.Web.UI.HtmlControls;
using System.Web.UI;
using System.Xml;

using RussellMason.Web.UI.Office2007.Design;

namespace RussellMason.Web.UI.Office2007
{

	/// <summary>
	/// Base class for Ribbon buttons.
	/// </summary>
	[
	DefaultEvent("Click")
	]
	public class RibbonButton : RibbonControl, IPostBackEventHandler
	{

		#region Static Properties

		/// <summary>
		/// Gets the Click event key.
		/// </summary>
		public static readonly object ClickEvent = new object();

		#endregion Static Properties

		#region Fields

		private string _script = string.Empty;
		private bool _autoPostBack = false;

		#endregion Fields

		#region Constructors

		/// <summary>
		/// Allows derived classes to create an instance of the RibbonButton class.
		/// </summary>
		protected RibbonButton()
		{
			// Shared State
			// 0 - Enabled
			// 1 - Checked
			// 2 - EnabledImageUrl
			// 3 - DisabledImageUrl
			this.SharedState.Add(new SharedStateItem("Enabled", typeof(bool), true));
			this.SharedState.Add(new SharedStateItem("Checked", typeof(bool), false));
			this.SharedState.Add(new SharedStateItem("EnabledImageUrl", typeof(string), string.Empty));
			this.SharedState.Add(new SharedStateItem("DisabledImageUrl", typeof(string), string.Empty));
		} 

		#endregion Constructors

		#region Properties

		/// <summary>
		/// Gets or sets whether the button is enabled or disabled.
		/// </summary>
		[
		DefaultValue(true),
		Description("Specifies whether the button is enabled or disabled."),
		Category("Behavior")
		]
		public override bool Enabled
		{
			get
			{
				bool enabled = Convert.ToBoolean(this.SharedState["Enabled"].Value);

				return enabled;
			}
			set
			{
				this.SharedState["Enabled"].Value = value;
			}
		}

		/// <summary>
		/// Gets or sets whether the button is in a checked state or not.
		/// </summary>
		[
		DefaultValue(false),
		Description("Specifies whether the button is in a checked state or not."),
		Category("Behavior")
		]
		public virtual bool Checked
		{
			get
			{
				bool isChecked = Convert.ToBoolean(this.SharedState["Checked"].Value);

				return isChecked;
			}
			set
			{
				this.SharedState["Checked"].Value = value;
			}
		}

		/// <summary>
		/// Gets or sets the image to display when enabled. 32x32 pixels.
		/// </summary>
		[
		Editor(typeof(ImageUrlEditor), typeof(UITypeEditor)),
		Description("Specifies the image to display when enabled. 32x32 pixels."),
		Category("Appearance")
		]
		public string EnabledImageUrl
		{
			get
			{
				string url = Convert.ToString(this.SharedState["EnabledImageUrl"].Value);

				if (string.IsNullOrEmpty(url))
				{
					this.SharedState["EnabledImageUrl"].Value = "~/Office2007Client/Images/Empty.gif";
					url = "~/Office2007Client/Images/Empty.gif";
				}

				return url;
			}
			set
			{
				if (string.IsNullOrEmpty(value))
				{
					value = "~/Office2007Client/Images/Empty.gif";
				}

				value = ResolveUrl(value);
				this.SharedState["EnabledImageUrl"].Value = value;
			}
		}

		/// <summary>
		/// Gets or sets the image to display when disabled. 32x32 pixels.
		/// </summary>
		[
		Editor(typeof(ImageUrlEditor), typeof(UITypeEditor)),
		Description("Specifies the image to display when disabled. 32x32 pixels."),
		Category("Appearance")
		]
		public string DisabledImageUrl
		{
			get
			{
				string url = Convert.ToString(this.SharedState["DisabledImageUrl"].Value);

				if (string.IsNullOrEmpty(url))
				{
					this.SharedState["DisabledImageUrl"].Value = "~/Office2007Client/Images/Empty.gif";
					url = "~/Office2007Client/Images/Empty.gif";
				}

				return url;
			}
			set
			{
				if (string.IsNullOrEmpty(value))
				{
					value = "~/Office2007Client/Images/Empty.gif";
				}

				value = ResolveUrl(value);
				this.SharedState["DisabledImageUrl"].Value = value;
			}
		}

		/// <summary>
		/// Gets or set the client code to execute when the button is clicked.
		/// </summary>
		[
		DefaultValue(""),
		Description("Specifies the client code to execute when the button is clicked."),
		Category("Behavior")
		]
		public virtual string OnClientClick
		{
			get
			{
				return _script;
			}
			set
			{
				_script = value;
			}
		}

		/// <summary>
		/// Gets or sets whether clicking the button causes a post back.
		/// </summary>
		[
		DefaultValue(false),
		Description("Specifies whether clicking the button causes a post back."),
		Category("Behavior")
		]
		public virtual bool AutoPostBack
		{
			get
			{
				return _autoPostBack;
			}
			set
			{
				_autoPostBack = value;
			}
		}

		/// <summary>
		/// Gets or sets whether the button causes validation to fire.
		/// </summary>
		[
		DefaultValue(true),
		Description("Specifies whether whether the button causes validation to fire."),
		Category("Behavior")
		]
		public virtual bool CausesValidation
		{
			get
			{
				bool causesValidation = true;

				if (this.ViewState["CausesValidation"] != null)
				{
					causesValidation = Convert.ToBoolean(this.ViewState["CausesValidation"]);
				}

				return causesValidation;
			}
			set
			{
				this.ViewState["CausesValidation"] = value;
			}
		}

		/// <summary>
		/// Gets or sets the group that should be validated when the control causes a Postback.
		/// </summary>
		[
		DefaultValue(""),
		Description("Specifies the group that should be validated when the control causes a Postback."),
		Category("Behavior")
		]
		public string ValidationGroup
		{
			get
			{
				string validationGroup = string.Empty;

				if (this.ViewState["ValidationGroup"] != null)
				{
					validationGroup = Convert.ToString(this.ViewState["ValidationGroup"]);
				}

				return validationGroup;
			}
			set
			{
				this.ViewState["ValidationGroup"] = value;
			}
		}

		/// <summary>
		/// Gets or sets the URL to navigate to when the button is clicked. JavaScript in the
		/// OnClientClick event is executed first, then navigation to the URL is performed. To 
		/// cancel the navigation return false from the OnClientClick event.
		/// </summary>
		[
		DefaultValue(""),
		Description("Specifies the URL to navigate to when the button is clicked."),
		Category("Behavior")
		]
		public string NavigateUrl
		{
			get
			{
				string navigateUrl = string.Empty;

				if (this.ViewState["NavigateUrl"] != null)
				{
					navigateUrl = Convert.ToString(this.ViewState["NavigateUrl"]);
				}

				return navigateUrl;
			}
			set
			{
				this.ViewState["NavigateUrl"] = value;
			}
		}

		/// <summary>
		/// Gets or sets the target used when the button is clicked. This has no affect unless the
		/// NavigateUrl property is also set.
		/// </summary>
		[
		DefaultValue(""),
		Description("Specifies the target used when the button is clicked. Has no affect unless NavigateUrl is also set."),
		Category("Behavior")
		]
		public string Target
		{
			get
			{
				string target = string.Empty;

				if (this.ViewState["Target"] != null)
				{
					target = Convert.ToString(this.ViewState["Target"]);
				}

				return target;
			}
			set
			{
				this.ViewState["Target"] = value;
			}
		}

		#endregion Properties

		#region Methods

		/// <summary>
		/// Sets image - allows use by control and designer.
		/// </summary>
		public override void FinalizeRender()
		{
			base.FinalizeRender();

			SetCheckedState();
			SetImage();

			if (!this.DesignMode)
			{
				RenderClientPostBack();
			}
		}

		/// <summary>
		/// Ensures a disabled attribute is not written.
		/// </summary>
		/// <param name="writer">
		/// The writer to write to.
		/// </param>
		public override void RenderBeginTag(HtmlTextWriter writer)
		{
			// Because we are using Enabled the base class will write a disabled attribute.
			// This prevents the Shared state from being accessed when disabled so we set it
			// to always be enabled so the attribute is not written, then reset once done.
			bool enabled = this.Enabled;
			this.Enabled = true;

			try
			{
				base.RenderBeginTag(writer);
			}
			finally
			{
				this.Enabled = enabled;
			}
		}

		#endregion Methods

		#region Events

		/// <summary>
		/// Raises the Click event.
		/// </summary>
		/// <param name="e">
		/// The event data.
		/// </param>
		protected virtual void OnClick(EventArgs e)
		{
			EventHandler clickHandler = this.Events[RibbonButton.ClickEvent] as EventHandler;

			if (clickHandler != null)
			{
				clickHandler(this, e);
			}
		}

		/// <summary>
		/// Occurs when the button is clicked.
		/// </summary>
		public event EventHandler Click
		{
			add
			{
				this.Events.AddHandler(RibbonButton.ClickEvent, value);
			}
			remove
			{
				this.Events.RemoveHandler(RibbonButton.ClickEvent, value);
			}
		}

		#endregion Events
		
		#region Implementation Properties

		/// <summary>
		/// Gets the control's tag type.
		/// </summary>
		protected override HtmlTextWriterTag TagKey
		{
			get
			{
				return HtmlTextWriterTag.Div;
			}
		}

		#endregion Implementation Properties

		#region Implementation Methods

		/// <summary>
		/// Create client script to cause a post back or execute client side java script.
		/// </summary>
		/// <param name="host">
		/// The host control to attach the onclick event to.
		/// </param>
		protected virtual void RenderClientPostBack()
		{
			string postBackReference = string.Empty;
			string clientCode = string.Empty;

			if (this.AutoPostBack)
			{
				PostBackOptions options = new PostBackOptions(this);
				options.AutoPostBack = true;
				options.Argument = this.ID;

				if ((this.CausesValidation) && (this.Page.GetValidators(this.ValidationGroup).Count > 0))
				{
					options.PerformValidation = true;
					options.ValidationGroup = this.ValidationGroup;
				}

				postBackReference = this.Page.ClientScript.GetPostBackEventReference(options);
			}

			if (!string.IsNullOrEmpty(this.OnClientClick))
			{
				clientCode = this.OnClientClick;
			}

			if (!string.IsNullOrEmpty(this.NavigateUrl))
			{
				string target = this.Target;

				if (string.IsNullOrEmpty(target))
				{
					target = "_self";
				}

				// Embedded JavaScript
				string windowOpenCode = 
					string.Format("window.open('{0}', '{1}');", this.NavigateUrl, target);

				clientCode += windowOpenCode;
			}

			if ((!string.IsNullOrEmpty(clientCode)) || (!string.IsNullOrEmpty(postBackReference)))
			{
				// Embedded JavaScript - Named dependency
				string code = "if (" + this.ClientMemberName + ".getEnabled()) {" + clientCode + postBackReference + "}";

				// Embedded JavaScript
				this.Attributes.Add("onclick", code);
			}
		}

		/// <summary>
		/// Sets the button's checked state.
		/// </summary>
		protected virtual void SetCheckedState()
		{
		}

		/// <summary>
		/// Sets the button's image depending on enabled/disabled state.
		/// </summary>
		protected virtual void SetImage()
		{
			HtmlImage image = this.FindControl("Image") as HtmlImage;

			if (image != null)
			{
				if (this.Enabled)
				{
					image.Src = ResolveUrl(this.EnabledImageUrl);
				}
				else
				{
					image.Src = ResolveUrl(this.DisabledImageUrl);
				}
			}
		}

		/// <summary>
		/// Writes the object's details to the specified writer.
		/// </summary>
		/// <param name="writer">
		/// The writer to write to.
		/// </param>
		protected override void SerializeContent(XmlWriter writer)
		{
			base.SerializeContent(writer);

			writer.WriteAttributeString("Checked", this.Checked.ToString());
			writer.WriteAttributeString("EnabledImageUrl", this.EnabledImageUrl);
			writer.WriteAttributeString("DisabledImageUrl", this.DisabledImageUrl);
		}

		/// <summary>
		/// Deserializes the object from the specified node.
		/// </summary>
		/// <param name="element">
		/// The element to get property values from.
		/// </param>
		public override void Deserialize(XmlElement element)
		{
			base.Deserialize(element);

			if (element.HasAttribute("Checked"))
			{
				string checkedState = element.Attributes["Checked"].Value;
				this.Checked = Convert.ToBoolean(checkedState);
			}

			if (element.HasAttribute("EnabledImageUrl"))
			{
				string enabledImageUrl = element.Attributes["EnabledImageUrl"].Value;
				this.EnabledImageUrl = enabledImageUrl;
			}

			if (element.HasAttribute("DisabledImageUrl"))
			{
				string disabledImageUrl = element.Attributes["DisabledImageUrl"].Value;
				this.DisabledImageUrl = disabledImageUrl;
			}
		}

		#endregion Implementation Methods

		#region IPostBackEventHandler Members

		/// <summary>
		/// Raises the click event;
		/// </summary>
		/// <param name="eventArgument">
		/// The event data.
		/// </param>
		public virtual void RaisePostBackEvent(string eventArgument)
		{
			OnClick(EventArgs.Empty);
		}

		#endregion IPostBackEventHandler Members

	}

}
